/*--------------------------------------------------------------------------
 * Copyright (C) 1992,1993,1994,1995,1996,1997,1998 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  TEMPVC32.C - This utility uses TMEX to view a read the temperature from 
 *               a DS1920/DS1820. It requires the Windows 95/NT TMEX
 *               drivers to be present.
 *
 * Compiler:  Microsoft Visual C 4.0
 * Version: 3.11Aalph
 *
 */

#define TMEXUTIL

#include "iBTMEXCW.H"   

/* Local Function Prototypes */
short FindFirstFamily(short, long);
short FindNextFamily(short, long);
uchar dowcrc(uchar);
short ReadTempScratch(long, uchar *);
short ReadTemperature(long, float far *);
void Exit_Prog(char *, short);

/* Global Variables to hold list information */
short PortType, PortNum;
uchar state_buffer[5120];
uchar CRC8;                   

/*----------------------------------------------------------------------
 *  This is the Main routine for TEMPVC.
 */
void main(short argc, char **argv)
{
   short printmsg=1,DoFirst=1,rslt;
   char tstr[80];       
   char ReturnString[256];
   long session_handle;                         
   float current_temp;
   HKEY GlobKey;
   DWORD dwType,size;

   /* check arguments to see if request instruction with '?' or too many */
   if ((argc > 2) || ((argc > 1) && (argv[1][0] == '?' || argv[1][1] == '?')))
   {
       printf("\nusage: tempvc32 <PortNum> \n"
              "  - read the temperature from a DS1920/DS1820 iButton\n"
              "  - <optional> argument 1 specifies the port number\n"
              "  - version 3.11Alpha\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* read the default PortNum and PortType from the registry */
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
         "Software\\Dallas Semiconductor\\iButton TMEX\\3.00",0,
         KEY_READ,&GlobKey) == ERROR_SUCCESS)
   {
      /* attempt to read the PortNum */
      size = 255;
      PortNum = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortNum",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortNum = atoi(ReturnString);
      }
      /* attempt to read the PortType */
      size = 255;
      PortType = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortType",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortType = atoi(ReturnString);
      }

      // close the key
      RegCloseKey(GlobKey);
   }
   else
     Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   /* check argument to see if provided port number */
   if (argc > 1)
       PortNum = atoi(argv[1]);

   /* check to see that indicated PortNum is  <= 15 */
   if (PortNum > 15)
     Exit_Prog("ERROR, Indicated PortNum is not valid!\n",
               -XONE_WIRE_PORT_ERROR);

   /* print a header */  
   printf("\nTEMPVC32 Version 3.11Alpha\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-1998\n\n");
   printf("Default Port number: %d    Port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("TMEX main driver: %s\n",tstr);
   printf("TMEX adapter driver #%d: ",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
      Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);
   printf(" %s\n\n\n",tstr);

   // loop to get a session handle and to a temperature conversion */
   for (;;)
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         /* setup the MicroLan */
         if (TMSetup(session_handle) == 1)
         {
            if (printmsg)
               printf("Searching for a DS1920/DS1820...\n\n");
            
            /* look for the first temperature iButton (family type 0x10) */
            if (DoFirst)
               rslt = FindFirstFamily(0x10,session_handle);
            else
               rslt = FindNextFamily(0x10,session_handle);

            if (rslt)
            {             
               printmsg = FALSE;
               DoFirst = FALSE;
    
               /* read the temperature of the found iButton */
               if (ReadTemperature(session_handle,&current_temp) == 1)                    
                  printf("Temperature: %6.1fC (%6.1fF)\n",current_temp,current_temp * 9 / 5 + 32);                                   
               /* failed to read the temperature */
               else
                  printf("Coversion failed!");
   
               MessageBeep(MB_ICONEXCLAMATION);
            }      
            /* Temperature iButton not on MicroLan */
            else   
            {
               /* check if not Temperature iButton's Found */
               if (DoFirst)
                  printf("DS1920/DS1820 not found on MicroLan!"); 
               break;
            }
         }   
         /* MicroLan port not valid */
         else   
         {
            printf("MicroLan setup failed!"); 
            break;
         }
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         printf("No Hardware Driver for this type found!");
         break;
      }
   }

   /* end the session if one is open */
   TMEndSession(session_handle);
}


/*----------------------------------------------------------------------
 * Find the first device with a particular family
 */
short FindFirstFamily(short family, long session_handle)
{
   short j;
   short ROM[9];
   
   /* set up to find the first device with family 'family */
   if (TMFamilySearchSetup(session_handle,&state_buffer,family) == 1)
   { 
      /* get first device in list with the specified family */
      if (TMNext(session_handle,state_buffer) == 1)
      {
         /* read the rom number */
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   
         
         /* check if correct type */
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
            printf("Serial ROM ID: ");  
            for (j = 7; j >= 0; j--)
               printf("%02X",ROM[j]);
            printf("    ");
            return 1;
         }        
      }
   }
     
   /* failed to find device of that type */
   return 0;                              
}


/*----------------------------------------------------------------------
 * Find the next device with a particular family.  Must call 
 * FindFirstFamily before this function.
 */
short FindNextFamily(short family, long session_handle)
{
   short j;
   short ROM[9];
   
   /* get first device in list with the specified family */
   if (TMNext(session_handle,state_buffer) == 1)
   {
      /* read the rom number */
      ROM[0] = 0;
      TMRom(session_handle,state_buffer,ROM);   
      
      /* check if correct type */
      if ((family & 0x7F) == (ROM[0] & 0x7F))
      {
         printf("Serial ROM ID: ");  
         for (j = 7; j >= 0; j--)
            printf("%02X",ROM[j]);
         printf("    ");
         return 1;
      }        
   }
     
   /* failed to find device of that type */
   return 0;                              
}


/*----------------------------------------------------------------------
 * Get a list of the files on the 'num' device in the ROM list.
 */                                                          
short ReadTemperature(long session_handle, float far *current_temp)
{
   short flag,tsht;
   float tmp,cr,cpc;
   uchar rbuf[10];       
   unsigned long st;
                         
   /* access the device */
   if (TMAccess(session_handle,&state_buffer) != 1)
      return 0;
         
   /* send the Recall E2 command make sure Scratch is correct */
   TMTouchByte(session_handle,0xB8);

   /* send the start T command */
   if (TMAccess(session_handle,&state_buffer) != 1)
      return 0;
   
   /* prepare the strong pullup after next TMTouchByte */
   TMOneWireLevel(session_handle,LEVEL_SET,LEVEL_STRONG_PULLUP,PRIMED_BYTE);
         
   /* send the temperature conversion command */
   TMTouchByte(session_handle,0x44);
         
   /* sleep for a second */
   st = GetTickCount() + 1000; 
   while (GetTickCount() < st)
      TMValidSession(session_handle);

   /* disable the strong pull-up */
   TMOneWireLevel(session_handle,LEVEL_SET,LEVEL_NORMAL,PRIMED_NONE);
   
   /* verify conversion is complete */
   if (TMTouchBit(session_handle,0x01) != 0x01)
      return -100; 
                         
   /* read the scratchpad */
   flag = ReadTempScratch(session_handle,rbuf);
   if (flag != 1)
      return 0;
         
   /* calculate the temperature  */      
   tsht = rbuf[0];
   if (rbuf[1] & 0x01)
      tsht |= -256;
   tmp = (float)(tsht/2);
   cr = rbuf[6];
   cpc = rbuf[7];
   if (rbuf[7] == 0)
      return 0;   
   else
      tmp = tmp - (float)0.25 + (cpc - cr)/cpc;
   
   *current_temp = tmp;
         
   return 1;   
}


/*------------------------------------------------------------------------
 * Read the scratch of a DS18/1920.
 */
short ReadTempScratch(long session_handle, uchar *buf)
{
   short i;
                         
   /* access device */
   if (TMAccess(session_handle,&state_buffer) != 1)
      return 0;
         
   /* send read scratch command */
   TMTouchByte(session_handle,0xBE);
                              
   /* zero the dowcrc */                          
   CRC8 = 0;
   
   /* read scratch */
   for (i = 0; i < 8; i++)   
   {
      buf[i] = (uchar)TMTouchByte(session_handle,(short)0xFF);
      dowcrc(buf[i]);
   }
   
   /* check crc */
   if (dowcrc((uchar)TMTouchByte(session_handle,(short)0xFF)) != 0)
      return 0;
      
   return 1;   
}                   


/*--------------------------------------------------------------------------
 * Update the Dallas Semiconductor One Wire CRC (CRC8) from the global
 * variable CRC8 and the argument.  Return the updated CRC8.
 */
uchar dscrc_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};

uchar dowcrc(uchar x)
{
   CRC8 = dscrc_table[CRC8 ^ x];
   return CRC8;
}

/*--------------------------------------------------------------------------
 *  Prints a message, beeps and exits the program.
 */
void Exit_Prog(char *msg, short errcd)
{
     /* print the message left justified */
     printf("\r%s",msg);

     /*  error beep if return code is not 0 */
     if (errcd != NORMAL_EXIT)
        MessageBeep(MB_ICONEXCLAMATION);

     /*  return error code passed */
     exit(errcd);
}

